from __future__ import unicode_literals

DATA_DIRNAME = "data"
DEFAULT_GLYPHS_DIRNAME = "glyphs"
IMAGES_DIRNAME = "images"

FEATURES_FILENAME = "features.fea"
FONTINFO_FILENAME = "fontinfo.plist"
GROUPS_FILENAME = "groups.plist"
KERNING_FILENAME = "kerning.plist"
LAYERCONTENTS_FILENAME = "layercontents.plist"
LIB_FILENAME = "lib.plist"
METAINFO_FILENAME = "metainfo.plist"

CONTENTS_FILENAME = "contents.plist"
LAYERINFO_FILENAME = "layerinfo.plist"

DEFAULT_LAYER_NAME = "public.default"
